library(dplyr)
library(BIEN)
library(dismo) # for gbif()
library(CoordinateCleaner)
library(httr)

# get north american extent
s_cont = shapefile('gis resources/continent/continent.shp')
s_na = s_cont[s_cont$CONTINENT=="North America",]
s_na = crop(s_na, extent(-180,0,0,100))


download_bien <- function(speciesname)
{
  data = BIEN_occurrence_species(speciesname, only.new.world = T, native.status = T, observation.type = T, cultivated = T, all.taxonomy = T)
  
  if (nrow(data) > 0)
  {
    data = data %>%
      dplyr::select(latitude,longitude) %>%
      na.omit
    
    if (nrow(data) > 0)
    {
      data$Provenance = 'BIEN'
      data$Species = speciesname
    }
    else
    {
      data = NULL
    }
  }
  else
  {
    data = NULL
  }
  
  return(data)
}

download_gbif <- function(speciesname)
{
  data <- gbif(genus=strsplit(speciesname, " ")[[1]][1], species=strsplit(speciesname, " ")[[1]][2], 
               download=T, geo=T, ext=extent(s_na))
  
  if (!is.null(data))
  {
    if ("lon" %in% names(data) & "lat" %in% names(data))
    {
      data = data %>% 
        filter(basisOfRecord == "PRESERVED_SPECIMEN") %>%
        rename(longitude=lon, latitude=lat) %>%
        filter(year > 1945) %>% # remove records from before second world war
        dplyr::select(latitude,longitude) %>%
        na.omit
      
      if (nrow(data) > 0)
      {
        data$Provenance = 'GBIF'
        data$Species = speciesname
      }
      else
      {
        data = NULL
      }
    }
    else
    {
      data = NULL
    }
  }
  else
  {
    data = NULL
  }
  
  return(data)
}

download_seinet <- function(speciesname)
{
  tmp <- tempfile()   
  url <- paste("http://swbiodiversity.org/seinet/webservices/dwc/dwcapubhandler.php?cond=sciname:",strsplit(speciesname," ")[[1]][1],"%20",strsplit(speciesname," ")[[1]][2],sep="")
  GET(url, write_disk(tmp))
  
  data <- NULL
  try(data <- read.csv(unz(tmp,'occurrences.csv')))
  
  if (!is.null(data))
  {
    data = data %>%
      rename(latitude=decimalLatitude, longitude=decimalLongitude) %>%
      dplyr::select(latitude, longitude) %>%
      na.omit
    
    if (nrow(data) > 0)
    {
      data$Provenance = 'SEInet'
      data$Species = speciesname
    }
    else
    {
      data = NULL
    }   
  }
  else
  {
    data = NULL
  }
  
  return(data)
}

get_data_for_species <- function(speciesname)
{
  print('BIEN...')
  data_bien = download_bien(speciesname)
  print(sprintf('%s - %d records\n', speciesname, ifelse(!is.null(data_bien),nrow(data_bien),0)))
  
  print('GBIF...')
  data_gbif = download_gbif(speciesname)
  print(sprintf('%s - %d records\n', speciesname, ifelse(!is.null(data_gbif),nrow(data_gbif),0)))
  
  print('SEInet...')
  data_seinet = download_seinet(speciesname)
  print(sprintf('%s - %d records\n', speciesname, ifelse(!is.null(data_seinet),nrow(data_seinet),0)))
  
  data_all = rbind(data_bien, data_gbif, data_seinet)
  
  return(data_all)
}


clean_data <- function(df) 
{
  nrow.start = nrow(df)
  
  df <- cc_val(df, lon="longitude", lat="latitude",value='clean') # check lat/lon in range
  df <- cc_equ(df,lon="longitude", lat="latitude",value='clean') # check lat/lon equal by mistake
  df <- cc_gbif(df,lon="longitude", lat="latitude",value='clean') # check GBIF hq
  df <- cc_inst(df,lon="longitude", lat="latitude",value='clean') # check gardens/herbaria 
  df <- cc_dupl(df,lon="longitude", lat="latitude",value='clean',species='Species') # check duplicates
  df <- cc_zero(df,lon="longitude", lat="latitude",value='clean') # check for zero coordinates
  
  # see if points are in north america
  df_spatial = df; coordinates(df_spatial) <- ~longitude + latitude
  proj4string(df_spatial) = proj4string(s_na)
  in.out = !is.na( over(df_spatial, s_na)$CONTINENT )
  df <- df[in.out,]
  
  nrow.finish = nrow(df)
  
  print(sprintf("Retained %d/%d rows (%.3f)",nrow.finish,nrow.start,nrow.finish/nrow.start))
  
  return(df)
}

# load species list
data_specieslist = read.csv('carolyn data/march.ensemble.NARP.splist.csv')

# get unique species
data_specieslist_vector = do.call("c",lapply(data_specieslist, as.character))
species_all = data_specieslist_vector %>% 
  unique %>% 
  na.omit %>% 
  sort
species_all = species_all[species_all!=""]

# iterate through all species
if (!file.exists('occurrence data'))
{
  dir.create('occurrence data')
}


finished_ids = as.numeric(sapply(strsplit(dir('occurrence data',pattern='DATA*'),split="\\*"),function(x){x[2]}))
ids_todo = setdiff(1:length(species_all), finished_ids)

for (i in ids_todo)
{
  print(sprintf("%d/%d - %s", i, length(species_all), species_all[i]))
  
  # backup data frame
  df_out = data.frame(longitude=NA,latitude=NA,Species=NA,Provenance=NA) %>% na.omit
  
  data_this = get_data_for_species(species_all[i])
  if (!is.null(data_this))
  {
    if (nrow(data_this) > 0)
    {
      data_this_cleaned = clean_data(data_this)
      if (!is.null(data_this_cleaned))
      {
        if (nrow(data_this_cleaned) > 0)
        {
          # replace blank data with cleaned data
          df_out = data_this_cleaned
        }
      }
    }
  }
  
  write.csv(df_out, file=sprintf('occurrence data/DATA*%d*%s.csv',i,species_all[i]),row.names=FALSE)
  
  # wait to avoid overloading APIs
  Sys.sleep(1)
}


# identify BIEN species in sonoran desert
s_sonorandesert = shapefile('gis resources/outputs/sonoran_desert.shp')
proj_latlon = "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
s_sonorandesert_latlon = spTransform(s_sonorandesert, proj_latlon)

data_specieslist_non_sonorandesert = BIEN_list_spatialpolygons(s_sonorandesert_latlon, cultivated = FALSE, only.new.world=TRUE)
data_specieslist_non_sonorandesert = as.character(data_specieslist_non_sonorandesert$scrubbed_species_binomial)
write.csv(data.frame(species_name=data_specieslist_non_sonorandesert),'carolyn data/specieslist_sonorandesert_bien.csv',row.names=F)
