# need to run with /usr/local/bin/Rscript-4.0.2 2\ do\ SDMs.R > log.txt 2>&1 & - 22907

library(dplyr)
library(dismo)
library(raster)
library(sdm) # check gm installed
library(parallel)

# iterate through all species
if (!file.exists('sdms'))
{
  dir.create('sdms')
}

# get list of species
species_list = dir('occurrence data',pattern='DATA*',full.names=TRUE)

# load in rasters
r_na_10km = raster('gis resources/outputs/r_na_10km.tif')
r_na_25km = raster('gis resources/outputs/r_na_25km.tif')
r_na_50km = raster('gis resources/outputs/r_na_50km.tif')
r_sonorandesert_10km = raster('gis resources/outputs/r_sonorandesert_masked_10km.tif')
r_sonorandesert_25km = raster('gis resources/outputs/r_sonorandesert_masked_25km.tif')
r_sonorandesert_50km = raster('gis resources/outputs/r_sonorandesert_masked_50km.tif')

r_climate_10km = stack(dir(path='gis resources/outputs',pattern='r_climate_10km*',full.names=TRUE))
names(r_climate_10km) = gsub("r_climate_10km_CHELSA_","",names(r_climate_10km))

r_climate_25km = stack(dir(path='gis resources/outputs',pattern='r_climate_25km*',full.names=TRUE))
names(r_climate_25km) = gsub("r_climate_25km_CHELSA_","",names(r_climate_25km))

r_climate_50km = stack(dir(path='gis resources/outputs',pattern='r_climate_50km*',full.names=TRUE))
names(r_climate_50km) = gsub("r_climate_50km_CHELSA_","",names(r_climate_50km))

proj_laea = projection(r_na_10km)


prepare_data <- function(df, r_mask, r_predictors, string_suffix, n_resample)
{
	species_name = df$Species[1]
	
	points = df %>% dplyr::select(longitude,latitude)
	coordinates(points) = ~longitude+latitude
	crs(points) = "+proj=longlat +datum=WGS84"
	
	# project data to UTM
	points_laea = spTransform(points, proj_laea)
	
	# keep only points in the mask
	in.out = which(!is.na(extract(r_mask, points_laea)))
  points_laea <- points_laea[in.out,]
  	
  # resample to the same number of points to control sampling effort
  points_laea_resampled = points_laea[sample(1:nrow(points_laea@coords),size=n_resample,replace=TRUE),]
	
	# resample to one point per grid cell
	points_laea_gridsampled = SpatialPoints(gridSample(points_laea_resampled, r_mask,n=1))
	crs(points_laea_gridsampled) = crs(points_laea_resampled)
	
	# count # of original and retained points
	n_raw = nrow(points@coords)
	n_resampled = nrow(points_laea_resampled@coords)
	n_gridsampled = nrow(points_laea_gridsampled@coords)
	
	# make random points
	points_random = SpatialPoints(randomPoints(mask=r_mask,
									n=n_gridsampled,
									p=points_laea_gridsampled,
									excludep=TRUE))
	crs(points_random) = crs(points_laea)
	
	# convert to pseudoabsence/presence data
	points_absence = SpatialPointsDataFrame(points_random,data.frame(PA=rep(0,n_gridsampled)))
	points_presence = SpatialPointsDataFrame(points_laea_gridsampled,data.frame(PA=rep(1,n_gridsampled)))
	points_final = rbind(points_absence, points_presence)
	
	# convert to SDM data format
	data_sdm = sdmData(formula=PA~., train=points_final, predictors=r_predictors)
			
	# make a map of the data
	png(file=sprintf('sdms/%s_%s_input.png',species_name, string_suffix),width=800,height=800)
	plot(r_mask,legend=FALSE,main=species_name,col=gray.colors(100))
	points(points_presence,col='red',pch=3)
	points(points_absence,col='blue',pch=4)
	points(points_laea,col='green') # show un-resampled points
	dev.off()

	# return data
	return(list(data_sdm=data_sdm,
		points_final=points_final,
		n_raw=n_raw, 
		n_resampled=n_resampled,
		n_gridsampled=n_gridsampled,
		species_name=species_name))
}

run_sdm <- function(filename, r_mask_na, r_mask_sonorandesert, r_predictors, string_suffix, n_occ_min, n_resample)
{
	data_this = read.csv(filename)
	
	if (nrow(data_this) >= n_occ_min)
	{
		# prepare data
		print('...preparing data')
		data_prep = prepare_data(data_this, r_mask_na, r_predictors, string_suffix, n_resample)
		species_name = data_prep$species_name
		
		if (data_prep$n_gridsampled >= n_occ_min)
		{
			# run SDMs
			print('...running SDMs')
			m_sdm = sdm(PA~., data= data_prep$data_sdm, methods=c('gam','gbm','rf'),replication='sub',test.percent=20,n=5) # change this
			
			# get stats
			mean_auc = mean(getEvaluation(m_sdm)$AUC)
			mean_cor = mean(getEvaluation(m_sdm)$COR)
			mean_tss = mean(getEvaluation(m_sdm)$TSS)
			sd_auc = sd(getEvaluation(m_sdm)$AUC)
			sd_cor = sd(getEvaluation(m_sdm)$COR)
			sd_tss = sd(getEvaluation(m_sdm)$TSS)
			
			# calculate ensemble
			print('...making ensemble')
			m_ensemble = ensemble(m_sdm, 
				newdata=r_predictors,
				filename=tempfile(),
				setting=list(method='weighted',stat='AUC'))
			# clip to mask
			m_ensemble = mask(m_ensemble, r_mask_na)	
			
			# generate map of R across north america
			m_R = r_mask_na
			m_R[!is.na(m_R[])] <- 0
			m_rasterized = sign(rasterize(SpatialPoints(data_prep$points_final[data_prep$points_final$PA==1,]),r_mask_na))
			m_rasterized[is.na(m_rasterized)] <- 0
			m_R = m_R + m_rasterized
			
			# generate map of P across north america
			# make maps of P
			# get observed vs predicted values
			print('...selecting thresholds')
			xy = coordinates(data_prep$points_final)
			pa_obs = data_prep$points_final$PA
			pa_pred = extract(m_ensemble, xy)
			
			m_ensemble_stats = evaluates(x=pa_obs, p=pa_pred)@threshold_based %>% 
			  filter(criteria %in% c("sp=se","max(se+sp)")) #se+sp is like kappa
			
			# apply each threshold to the masked ensemble prediction and make a stack
			m_ensemble_thresholded_P = lapply(1:nrow(m_ensemble_stats), function(i) { 
			  m_ensemble_binary = (m_ensemble > m_ensemble_stats$threshold[i])
			  return(m_ensemble_binary)
			})
			
      # mask results to sonoran desert
			print('...estimating R and P')
			
			# project and mask (because mask may not have same extent/alignment as ensemble)
      m_R_sonorandesert = projectRaster(m_R, r_mask_sonorandesert)
      m_R_sonorandesert = mask(m_R_sonorandesert, r_mask_sonorandesert)
      m_R_sonorandesert = sign(m_R_sonorandesert) # take anything above zero

			m_P_sonorandesert = lapply(m_ensemble_thresholded_P, projectRaster, r_mask_sonorandesert)
			m_P_sonorandesert = lapply(m_P_sonorandesert, mask, r_mask_sonorandesert)
			m_P_sonorandesert = lapply(m_P_sonorandesert, sign) # take anything above zero
			
			# calculate R and P in km2
			stat_R_sonorandesert = length(which(m_R_sonorandesert[]==1)) * prod(res(m_R_sonorandesert)) / 1000^2
			stat_P_sonorandesert = sapply(m_P_sonorandesert, function(x) { 
				length(which(x[]==1)) * prod(res(x)) / 1000^2
				})
				
			# make a table
			table_out = m_ensemble_stats
			table_out$n_raw = data_prep$n_raw
			table_out$n_resampled = data_prep$n_resampled
			table_out$n_gridsampled = data_prep$n_gridsampled
			table_out$species_name = species_name
			table_out$suffix = string_suffix
			table_out$stat_R = stat_R_sonorandesert
			table_out$stat_P = stat_P_sonorandesert
			table_out$stat_R_over_P = stat_R_sonorandesert / stat_P_sonorandesert
			table_out$mean_auc = mean_auc
			table_out$mean_cor = mean_cor
			table_out$mean_tss = mean_tss
			table_out$sd_auc = sd_auc
			table_out$sd_cor = sd_cor
			table_out$sd_tss = sd_tss
			
			print('...writing outputs')
			# make outputs
			m_R_out = m_R_sonorandesert
			names(m_R_out) = "R"
			m_P_out = stack(m_P_sonorandesert)
			names(m_P_out) = paste("P",m_ensemble_stats$criteria)
			# R
			writeRaster(m_R_out,
			            file=sprintf('sdms/%s_%s_R.tif',species_name, string_suffix),overwrite=TRUE)
			# P 
			writeRaster(m_P_out,
				file=sprintf('sdms/%s_%s.tif',species_name, string_suffix),
				bylayer=TRUE,overwrite=TRUE,suffix='names')
				
			# image
			png(file=sprintf('sdms/%s_%s_output.png',species_name, string_suffix),width=1000,height=1000)
			stack_final = stack(c(m_R_out, m_P_out))
			plot(stack_final)
			dev.off()	
			
			# table
			write.csv(table_out, row.names=F, file=sprintf('sdms/%s_%s_table.csv',species_name, string_suffix))
			
			# sdm object
			#saveRDS(m_sdm, file=sprintf('sdms/%s_%s_sdm.Rdata',species_name, string_suffix))
			
			return(m_sdm)
		} 
		else {
			print(sprintf("Not enough subsampled data for %s", filename))
			return(NULL)
		}
	}
	else {
		print(sprintf("Not enough raw data for %s", filename))
		return(NULL)
	}
}

# set random seed
set.seed(1)

# iterate through all species
for (i in 1:length(species_list))
#mclapply(X=1:length(species_list), mc.cores=8, mc.set.seed=FALSE, FUN=function(i)  # 
{
 	# replicate independent data
  	#for (j in 1:5) # 5 resamples per pipeline
    mclapply(X=1:5, mc.cores=5, mc.set.seed=TRUE, FUN=function(j)
  	{
  		print(sprintf("%d/%d - %d - %s", i, length(species_list), j, species_list[i]))
  		
  		# do 50 km analysis
  	  #result_50km = NULL
	  	#result_50km <- try(run_sdm(filename=species_list[i], 
  		#  r_mask_na = r_na_50km, 
  		#  r_mask_sonorandesert = r_sonorandesert_50km, 
  		#  r_predictors = r_climate_50km, 
  		#  string_suffix = sprintf("%s_rep%d","50km",j),
  		#  n_occ_min = 5, n_resample=500))
	  	
  		#if (is.null(result_50km))
  		#{
  		#	print(sprintf("Error with %s",species_list[i]))
  		#}
	  	
	  	# do 50 km analysis
	  	result_25km = NULL
	  	result_25km <- try(run_sdm(filename=species_list[i], 
	  	                           r_mask_na = r_na_25km, 
	  	                           r_mask_sonorandesert = r_sonorandesert_25km, 
	  	                           r_predictors = r_climate_25km, 
	  	                           string_suffix = sprintf("%s_rep%d","25km",j),
	  	                           n_occ_min = 5, n_resample=500))
	  	
	  	if (is.null(result_25km))
	  	{
	  	  print(sprintf("Error with %s",species_list[i]))
	  	}
	 }
  )
}
#)
