library(dplyr)
library(raster)
library(data.table)
library(ggplot2)
library(tidyr)
library(wesanderson)
library(car)
library(visreg)
library(ggpubr)
library(V.PhyloMaker)
library(nlme)

files50 = dir('sdms50km', pattern='*csv$', full.names=TRUE)
files25 = dir('sdms25km', pattern='*csv$', full.names=TRUE)

results_all = lapply(c(files50, files25), read.csv)
results_all = rbindlist(results_all)

results_all$scale = sapply(strsplit(as.character(results_all$suffix),split="_"),head,1)
results_all$scale = as.numeric(gsub("km","",results_all$scale))

results_all$replicate = sapply(strsplit(as.character(results_all$suffix),split="_"),tail,1)
results_all$replicate = as.numeric(gsub("rep","",results_all$replicate))

results_all = results_all %>% dplyr::select(-suffix)

# write out File S2
write.csv(results_all, file='File S2 - sdm_results.csv',row.names=F)

# count occurrences
mean(results_all$n_raw)
sd(results_all$n_raw)

mean(results_all$n_gridsampled)
sd(results_all$n_gridsampled)

# load in species information
specieslist = read.csv('carolyn data/specieslist_sonorandesert_short.csv')

# prepare data for food plants
results_sdm_foodplants = results_all %>% 
  filter(species_name %in% specieslist$Food.species)
names(results_sdm_foodplants) = paste("foodplant",names(results_sdm_foodplants),sep=".")

results_sdm_foodplants = results_sdm_foodplants %>%
  rename(Food.species=foodplant.species_name)

# prepare data for contrasts
results_sdm_contrasts = results_all %>%
  filter(species_name %in% specieslist$Contrast.species)
names(results_sdm_contrasts) = paste("contrast",names(results_sdm_contrasts),sep=".")

results_sdm_contrasts = results_sdm_contrasts %>%
  rename(Contrast.species=contrast.species_name)

do_merge_at_scale <- function(scale)
{
    specieslist_merged = specieslist %>% 
      left_join(results_sdm_foodplants %>% filter(foodplant.scale==scale), by='Food.species') %>%
      left_join(results_sdm_contrasts %>% filter(contrast.scale==scale), by='Contrast.species')
    
    # filter data
    specieslist_merged_filtered = specieslist_merged %>%
      filter(foodplant.mean_auc >= 0.8 & contrast.mean_auc >= 0.8) %>% # include only 'good' SDMs
      filter(foodplant.n_gridsampled >= 5 & contrast.n_gridsampled >= 5) %>% # only species with sufficient data
      filter(Contrast.in.Sonoran.Desert == TRUE) %>% # only include contrasts that have at least some sonoran desert range
      filter(foodplant.stat_R > 0 & contrast.stat_R > 0) %>% # only include models that put species in
      filter(foodplant.stat_R_over_P <= 1 & contrast.stat_R_over_P <= 1) # only include models with R<P
    
    # add delta R/P
    specieslist_merged_filtered = specieslist_merged_filtered %>%
      mutate(delta_R_over_P = foodplant.stat_R_over_P - contrast.stat_R_over_P) %>% 
      mutate(Contrast.used=factor(Contrast.used,levels=c(0,1),labels=c("Unused","Used"))) %>%
      mutate(Contrast.type=ifelse(Contrast.type=="Sister","Sister","Congener"))
    
    return(specieslist_merged_filtered)
}

results_25km = do_merge_at_scale(25)
results_50km = do_merge_at_scale(50)




# do couts on runs
data_for_counting = results_all %>% 
  ungroup %>%
  group_by(scale, criteria, species_name) %>%
  filter(mean_auc >= 0.8 & n_gridsampled >=5 & stat_R > 0 & stat_R_over_P <= 1)

# number of runs per species
data_for_counting_summary = data_for_counting %>% group_by(species_name) %>% summarize(n.models=n())
mean(data_for_counting_summary$n.models)
sd(data_for_counting_summary$n.models)


med_values_50km = results_50km %>%
  group_by(Contrast.used, Contrast.type) %>% #, foodplant.criteria, contrast.criteria
  summarize(median_delta_R_over_P=median(delta_R_over_P)) %>%
  ungroup

med_values_25km = results_25km %>%
  group_by(Contrast.used, Contrast.type) %>%
  summarize(median_delta_R_over_P=median(delta_R_over_P)) %>%
  ungroup
















# compare aggregate R/P
data_rp_food_25km = results_25km %>% 
  dplyr::select(RP=foodplant.stat_R_over_P,Species=Food.species) %>%
  mutate(Type="Food plant",Contrast.type="(Food plant)")
data_rp_used_25km = results_25km %>% 
  filter(Contrast.used=="Used") %>%
  dplyr::select(RP=contrast.stat_R_over_P,Contrast.type,Species=Contrast.species) %>%
  mutate(Type="Used contrast")
data_rp_unused_25km = results_25km %>% 
  filter(Contrast.used=="Unused") %>%
  dplyr::select(RP=contrast.stat_R_over_P,Contrast.type,Species=Contrast.species) %>%
  mutate(Type="Unused contrast")

data_rp_25km = rbind(data_rp_food_25km, data_rp_unused_25km, data_rp_used_25km)

data_rp_medians_25km = data_rp_25km %>%
  group_by(Contrast.type, Type) %>%
  summarize(median_RP=median(RP)) %>%
  ungroup

g_rp_raw_25km = ggplot(data_rp_25km %>% rename(Usage=Type,Relationship=Contrast.type), aes(x=RP,fill=Usage,col=Usage,linetype=Relationship)) + 
  geom_density(alpha=0.25) +
  xlim(0,1) +
  theme_bw() +
  scale_color_manual(values=wes_palette("Darjeeling1")) +
  scale_fill_manual(values=wes_palette("Darjeeling1")) +
  geom_vline(data = data_rp_medians_25km, aes(xintercept = median_RP,color=Type,linetype=Contrast.type),size=1,alpha=0.75) +
  scale_x_sqrt() +
  ylab("Density") +
  xlab("R/P")
ggsave(g_rp_raw_25km, file='g_rp_raw_25km.pdf',width=5,height=3)


data_for_raw_analysis = data_rp_25km %>% 
  rename(Usage=Type) %>% 
  group_by(Species, Usage, Contrast.type) %>% 
  summarize(RP=mean(RP))

m_rp_raw = lm(log10(RP)~Usage+Contrast.type,data=data_for_raw_analysis)
summary(m_rp_raw)
anova(m_rp_raw)



# compare delta R/P for each type of comparison  
g_rp_genus_25km = ggplot(results_25km, aes(x=delta_R_over_P,fill=Contrast.used,col=Contrast.used)) + 
  geom_vline(xintercept = 0,color='black') +
  geom_density(alpha=0.25) +
  theme_bw() +
  geom_vline(data = med_values_25km, aes(xintercept = median_delta_R_over_P,col=Contrast.used),size=1,alpha=0.75) +
  facet_wrap(~Contrast.type) +
  scale_color_manual(values=wes_palette("GrandBudapest1"),name='Contrast usage') +
  scale_fill_manual(values=wes_palette("GrandBudapest1"),name='Contrast usage') +
  xlim(-0.5,0.5) +
  ylab("Density") +
  xlab(expression(Delta*(R/P)~(food-contrast)))
ggsave(g_rp_genus_25km,file='g_rp_genus_25km.pdf',width=7,height=3)


# do linear model
data_for_genus_analysis = results_25km %>% 
  dplyr::select(delta_R_over_P,Contrast.used,Contrast.type,Food.species,Contrast.species) %>%
  group_by(Contrast.used,Contrast.type,Food.species,Contrast.species) %>%
  summarize(delta_R_over_P=mean(delta_R_over_P))

m_rp_genus = lm(delta_R_over_P~Contrast.used+Contrast.type,data=data_for_genus_analysis)
summary(m_rp_genus)
anova(m_rp_genus)









# CHECK POSSIBLE CONFLATING FACTORS

# see if delta R/P varies with n_gridsample or AUC
data_for_conflating_analysis_25km = rbind(results_25km %>% 
                              dplyr::select(stat_R_over_P=foodplant.stat_R_over_P, n_gridsampled=foodplant.n_gridsampled, scale=foodplant.scale, criteria=foodplant.criteria, auc=foodplant.mean_auc) %>%
                              mutate(scale=as.character(scale), criteria=as.character(criteria), Type='Food species'), 
                              results_25km %>% 
                              dplyr::select(stat_R_over_P=contrast.stat_R_over_P, n_gridsampled=contrast.n_gridsampled, scale=contrast.scale, criteria=contrast.criteria,Type=Contrast.used,auc=contrast.mean_auc) %>%
                              mutate(scale=as.character(scale), criteria=as.character(criteria))) 

# see if delta R/P varies with n_gridsample or AUC
data_for_conflating_analysis_50km = rbind(results_50km %>% 
                                            dplyr::select(stat_R_over_P=foodplant.stat_R_over_P, n_gridsampled=foodplant.n_gridsampled, scale=foodplant.scale, criteria=foodplant.criteria, auc=foodplant.mean_auc) %>%
                                            mutate(scale=as.character(scale), criteria=as.character(criteria), Type='Food species'), 
                                          results_50km %>% 
                                            dplyr::select(stat_R_over_P=contrast.stat_R_over_P, n_gridsampled=contrast.n_gridsampled, scale=contrast.scale, criteria=contrast.criteria,Type=Contrast.used,auc=contrast.mean_auc) %>%
                                            mutate(scale=as.character(scale), criteria=as.character(criteria))) 

data_for_conflating_analysis = rbind(data_for_conflating_analysis_25km, data_for_conflating_analysis_50km) %>%
  mutate(scale=paste(scale, "km"))


conflating_ngridsampled=data_for_conflating_analysis %>% 
  group_by(scale, criteria) %>% 
  do(data.frame(r2=summary(lm(stat_R_over_P~n_gridsampled,data=.))$r.squared))
mean(conflating_ngridsampled$r2)
sd(conflating_ngridsampled$r2)

conflating_auc=data_for_conflating_analysis %>% 
  group_by(scale, criteria) %>% 
  do(data.frame(r2=summary(lm(stat_R_over_P~auc,data=.))$r.squared))
mean(conflating_auc$r2)
sd(conflating_auc$r2)


g_gridsample = ggplot(data_for_conflating_analysis %>% unique, aes(x=n_gridsampled,y=stat_R_over_P,col=Type)) + 
  geom_point(alpha=0.1) +
  facet_wrap(~criteria+scale) + 
  theme_bw() +
  xlab("Number of grid-sampled points") +
  ylab("R/P") +
  stat_smooth() +
  scale_color_manual(values=wes_palette("Darjeeling1"),name='Usage') 
ggsave(g_gridsample, file='g_gridsample.pdf',width=7,height=6)

g_auc = ggplot(data_for_conflating_analysis %>% unique, aes(x=auc,y=stat_R_over_P,col=Type)) + 
  geom_point(alpha=0.1) +
  facet_wrap(~criteria+scale) + 
  theme_bw() +
  xlab("AUC statistic") +
  ylab("R/P") +
  stat_smooth() +
  scale_color_manual(values=wes_palette("Darjeeling1"),name='Usage') 
ggsave(g_auc, file='g_auc.pdf',width=7,height=6)




stats_byscale = results_all %>% 
  ungroup %>%
  group_by(scale, criteria, species_name) %>%
  filter(mean_auc >= 0.8 & n_gridsampled >=5 & stat_R > 0 & stat_R_over_P <= 1) %>%
  dplyr::select(stat_R_over_P, stat_R, stat_P) %>%
  summarize(stat_R_over_P=mean(stat_R_over_P,na.rm=T), stat_R=mean(stat_R,na.rm=T),stat_P=mean(stat_P,na.rm=T)) %>%
  gather(key='stat',value='value',c(stat_R_over_P, stat_R, stat_P)) %>% 
  pivot_wider(names_from=scale,values_from=value,names_prefix="km") %>%
  mutate(stat=factor(stat,levels=c("stat_R","stat_P","stat_R_over_P"),labels=c("R","P","R/P"),ordered=TRUE))

# summarize r2
r2_byscale = stats_byscale %>% 
  group_by(stat,criteria) %>% 
  do(data.frame(r2=summary(lm(km50~km25,data=.))$r.squared))
mean(r2_byscale$r2)
sd(r2_byscale$r2)

g_grainsize = ggplot(stats_byscale, aes(x=km25, y=km50,col=criteria)) +
  geom_point(alpha=0.25) +
  facet_wrap(~stat,scales='free') +
  geom_smooth(method='lm') +
  theme_bw() +
  scale_color_brewer(palette='Set1',name='Threshold') +
  xlab(expression(paste("Area (km"^2,", 25 km spatial grain)"))) +
  ylab(expression(paste("Area (km"^2,", 50 km spatial grain)"))) + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))
ggsave(g_grainsize, file='g_grainsize.pdf',width=8,height=3)



# do a trait analysis
data_traits = read.csv('carolyn data/8.12.traits.csv',na.strings=c("","NA")) %>%
  rename(Species=speciesname)

table(!is.na(data_traits$Polyploidy)) / nrow(data_traits)








data_rp_25km_with_traits = data_rp_25km %>%
  group_by(Species, Type, Contrast.type) %>%
  summarize(RP.mean=mean(RP)) %>% 
  left_join(data_traits,by='Species') %>%
  mutate(PerennialSimple=factor(Duration %in% c("perennial","annual/perennial"),levels=c(FALSE,TRUE),labels=c("Annual","Perennial"))) %>%
  mutate(Polyploidy=factor(Polyploidy,levels=c(0,1),labels=c("Diploid","Polyploid"))) %>%
  rename(Usage=Type) %>%
  ungroup %>%
  dplyr::select(-Contrast.type) %>%
  group_by(Species) %>%
  # since some species appear more than once in the above dataset, label them with their lowest level (i.e. food plant takes priority over contrast)
  mutate(Usage = factor(Usage,levels=c("Food plant","Used contrast","Unused contrast"),ordered=TRUE)) %>%
  slice(which.min(Usage)) %>%
  mutate(Usage = as.character(Usage))

# make a phylogeny
data_phylo_species_25km = unique(data_rp_25km_with_traits$Species)
write.table(data_phylo_species_25km,file="data_phylo_species_25km.csv",row.names=F)
# run through http://tnrs.iplantcollaborative.org/TNRSapp.html and name tnrs_results_25km.txt
data_phylo_tnrs = read.csv('tnrs_results_25km.txt',sep="\t") %>%
  dplyr::select(Name_submitted, Accepted_family) %>%
  mutate(species=Name_submitted,
         genus=sapply(strsplit(as.character(Name_submitted), " "),head,1),
         family=Accepted_family) %>%
  dplyr::select(species,genus,family)

tree_25km = phylo.maker(data_phylo_tnrs,scenarios='S3')$scenario.3
write.tree(tree_25km,file='carolyn data/sonorandesert_phylogeny_25km.new')

data_rp_25km_with_traits_for_analysis = data_rp_25km_with_traits %>% 
                                          dplyr::select(Usage, Polyploidy, PerennialSimple, Form, RP.mean) %>%
                                          na.omit %>%
                                          ungroup %>%
                                          mutate(Species=gsub(" ","_",Species)) %>%
                                          as.data.frame

row.names(data_rp_25km_with_traits_for_analysis) = data_rp_25km_with_traits_for_analysis$Species

tree_for_analysis_25km = keep.tip(tree_25km, row.names(data_rp_25km_with_traits_for_analysis))

m_traits_25km_phylo = gls(log10(RP.mean)~Usage*(Polyploidy+PerennialSimple+Form),
                   data=data_rp_25km_with_traits_for_analysis,
                   correlation=corBrownian(phy=tree_for_analysis_25km))

m_traits_25km_phylo_intercept = gls(log10(RP.mean)~1,
                          data=data_rp_25km_with_traits_for_analysis,
                          correlation=corBrownian(phy=tree_for_analysis_25km))

summary(m_traits_25km_phylo)
anova(m_traits_25km_phylo)

R2.n_25km <- 1-(as.numeric(logLik(m_traits_25km_phylo)/logLik(m_traits_25km_phylo_intercept)))


g_trait1_25km = visreg(m_traits_25km_phylo,xvar="Usage",by="Polyploidy",trans=exp,gg=TRUE,rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))
g_trait2_25km = visreg(m_traits_25km_phylo,xvar="Usage",by="PerennialSimple",trans=exp,gg=TRUE, rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))
g_trait3_25km = visreg(m_traits_25km_phylo,xvar="Usage",by="Form",trans=exp,gg=TRUE, rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))

g_traits_25km = ggarrange(g_trait1_25km, g_trait2_25km, g_trait3_25km,nrow=1,ncol=3,common.legend = TRUE,labels="AUTO")
ggsave(g_traits_25km,file='g_traits_25km.pdf',width=9,height=4)







# do bigger scale, 50 km

# compare aggregate R/P
data_rp_food_50km = results_50km %>% 
  dplyr::select(RP=foodplant.stat_R_over_P,Species=Food.species) %>%
  mutate(Type="Food plant",Contrast.type="(Food plant)")
data_rp_used_50km = results_50km %>% 
  filter(Contrast.used=="Used") %>%
  dplyr::select(RP=contrast.stat_R_over_P,Contrast.type,Species=Contrast.species) %>%
  mutate(Type="Used contrast")
data_rp_unused_50km = results_50km %>% 
  filter(Contrast.used=="Unused") %>%
  dplyr::select(RP=contrast.stat_R_over_P,Contrast.type,Species=Contrast.species) %>%
  mutate(Type="Unused contrast")

data_rp_50km = rbind(data_rp_food_50km, data_rp_unused_50km, data_rp_used_50km)

data_rp_medians_50km = data_rp_50km %>%
  group_by(Contrast.type, Type) %>%
  summarize(median_RP=median(RP)) %>%
  ungroup

g_rp_raw_50km = ggplot(data_rp_50km %>% rename(Usage=Type,Relationship=Contrast.type), aes(x=RP,fill=Usage,col=Usage,linetype=Relationship)) + 
  geom_density(alpha=0.25) +
  xlim(0,1) +
  theme_bw() +
  scale_color_manual(values=wes_palette("Darjeeling1")) +
  scale_fill_manual(values=wes_palette("Darjeeling1")) +
  geom_vline(data = data_rp_medians_50km, aes(xintercept = median_RP,color=Type,linetype=Contrast.type),size=1,alpha=0.75) +
  scale_x_sqrt() +
  ylab("Density") +
  xlab("R/P")
ggsave(g_rp_raw_50km, file='g_rp_raw_50km.pdf',width=5,height=3)


data_for_raw_analysis = data_rp_50km %>% 
  rename(Usage=Type) %>% 
  group_by(Species, Usage, Contrast.type) %>% 
  summarize(RP=mean(RP))

m_rp_raw = lm(log10(RP)~Usage+Contrast.type,data=data_for_raw_analysis)
summary(m_rp_raw)
anova(m_rp_raw)



# compare delta R/P for each type of comparison  
g_rp_genus_50km = ggplot(results_50km, aes(x=delta_R_over_P,fill=Contrast.used,col=Contrast.used)) + 
  geom_vline(xintercept = 0,color='black') +
  geom_density(alpha=0.25) +
  theme_bw() +
  geom_vline(data = med_values_50km, aes(xintercept = median_delta_R_over_P,col=Contrast.used),size=1,alpha=0.75) +
  facet_wrap(~Contrast.type) +
  scale_color_manual(values=wes_palette("GrandBudapest1"),name='Contrast usage') +
  scale_fill_manual(values=wes_palette("GrandBudapest1"),name='Contrast usage') +
  xlim(-0.5,0.5) +
  ylab("Density") +
  xlab(expression(Delta*(R/P)~(food-contrast)))
ggsave(g_rp_genus_50km,file='g_rp_genus_50km.pdf',width=7,height=3)










g_rp_raw_50km = ggplot(data_rp_50km %>% rename(Usage=Type,Relationship=Contrast.type), aes(x=RP,fill=Usage,col=Usage,linetype=Relationship)) + 
  geom_density(alpha=0.25) +
  xlim(0,1) +
  theme_bw() +
  scale_color_manual(values=wes_palette("Darjeeling1")) +
  scale_fill_manual(values=wes_palette("Darjeeling1")) +
  geom_vline(data = data_rp_medians_25km, aes(xintercept = median_RP,color=Type,linetype=Contrast.type),size=1,alpha=0.75) +
  scale_x_sqrt() +
  ylab("Density") +
  xlab("R/P")
ggsave(g_rp_raw_50km, file='g_rp_raw_50km.pdf',width=5,height=3)



data_rp_50km_with_traits = data_rp_50km %>%
  group_by(Species, Type, Contrast.type) %>%
  summarize(RP.mean=mean(RP)) %>% 
  left_join(data_traits,by='Species') %>%
  mutate(PerennialSimple=factor(Duration %in% c("perennial","annual/perennial"),levels=c(FALSE,TRUE),labels=c("Annual","Perennial"))) %>%
  mutate(Polyploidy=factor(Polyploidy,levels=c(0,1),labels=c("Diploid","Polyploid"))) %>%
  rename(Usage=Type) %>%
  ungroup %>%
  dplyr::select(-Contrast.type) %>%
  group_by(Species) %>%
  # since some species appear more than once in the above dataset, label them with their lowest level (i.e. food plant takes priority over contrast)
  mutate(Usage = factor(Usage,levels=c("Food plant","Used contrast","Unused contrast"),ordered=TRUE)) %>%
  slice(which.min(Usage)) %>%
  mutate(Usage = as.character(Usage))

# make a phylogeny
data_phylo_species_50km = unique(data_rp_50km_with_traits$Species)
write.table(data_phylo_species_50km,file="data_phylo_species_50km.csv",row.names=F)
# run through http://tnrs.iplantcollaborative.org/TNRSapp.html and name tnrs_results_50km.txt
data_phylo_tnrs = read.csv('tnrs_results_50km.txt',sep="\t") %>%
  dplyr::select(Name_submitted, Accepted_family) %>%
  mutate(species=Name_submitted,
         genus=sapply(strsplit(as.character(Name_submitted), " "),head,1),
         family=Accepted_family) %>%
  dplyr::select(species,genus,family)

tree_50km = phylo.maker(data_phylo_tnrs,scenarios='S3')$scenario.3
write.tree(tree_50km,file='carolyn data/sonorandesert_phylogeny_50km.new')

data_rp_50km_with_traits_for_analysis = data_rp_50km_with_traits %>% 
  dplyr::select(Usage, Polyploidy, PerennialSimple, Form, RP.mean) %>%
  na.omit %>%
  ungroup %>%
  mutate(Species=gsub(" ","_",Species)) %>%
  as.data.frame

row.names(data_rp_50km_with_traits_for_analysis) = data_rp_50km_with_traits_for_analysis$Species

tree_for_analysis_50km = keep.tip(tree_50km, row.names(data_rp_50km_with_traits_for_analysis))

m_traits_50km_phylo = gls(log10(RP.mean)~Usage*(Polyploidy+PerennialSimple+Form),
                          data=data_rp_50km_with_traits_for_analysis,
                          correlation=corBrownian(phy=tree_for_analysis_50km))

summary(m_traits_50km_phylo)
anova(m_traits_50km_phylo)


g_trait1_50km = visreg(m_traits_50km_phylo,xvar="Usage",by="Polyploidy",trans=exp,gg=TRUE,rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))
g_trait2_50km = visreg(m_traits_50km_phylo,xvar="Usage",by="PerennialSimple",trans=exp,gg=TRUE, rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))
g_trait3_50km = visreg(m_traits_50km_phylo,xvar="Usage",by="Form",trans=exp,gg=TRUE, rug=FALSE) + theme_bw() + ylab("Effect on R/P") + xlab("Usage category") + ylim(0,1) + theme(axis.text.x = element_text(angle = 45,hjust=1))

g_traits_50km = ggarrange(g_trait1_50km, g_trait2_50km, g_trait3_50km,nrow=1,ncol=3,common.legend = TRUE,labels="AUTO")
ggsave(g_traits_50km,file='g_traits_50km.pdf',width=9,height=4)
















