library(dplyr)
library(dismo)
library(raster)
library(sdm)
library(data.table)
library(ggplot2)
library(rgdal)
library(ggpubr)

s_sonorandesert = readOGR('gis resources/outputs/sonoran_desert.shp')
df_s_sonorandesert = fortify(s_sonorandesert,group=NA_L3CODE)

# get list of species
species_list = dir('occurrence data',pattern='DATA*',full.names=TRUE)

# load in rasters
r_na_10km = raster('gis resources/outputs/r_na_10km.tif')
r_na_25km = raster('gis resources/outputs/r_na_25km.tif')
r_na_50km = raster('gis resources/outputs/r_na_50km.tif')
r_sonorandesert_10km = raster('gis resources/outputs/r_sonorandesert_masked_10km.tif')
r_sonorandesert_25km = raster('gis resources/outputs/r_sonorandesert_masked_25km.tif')
r_sonorandesert_50km = raster('gis resources/outputs/r_sonorandesert_masked_50km.tif')

r_climate_10km = stack(dir(path='gis resources/outputs',pattern='r_climate_10km*',full.names=TRUE))
names(r_climate_10km) = gsub("r_climate_10km_CHELSA_","",names(r_climate_10km))

r_climate_25km = stack(dir(path='gis resources/outputs',pattern='r_climate_25km*',full.names=TRUE))
names(r_climate_25km) = gsub("r_climate_25km_CHELSA_","",names(r_climate_25km))

r_climate_50km = stack(dir(path='gis resources/outputs',pattern='r_climate_50km*',full.names=TRUE))
names(r_climate_50km) = gsub("r_climate_50km_CHELSA_","",names(r_climate_50km))

proj_laea = projection(r_na_10km)


prepare_data <- function(df, r_mask, r_predictors, string_suffix, n_resample)
{
  species_name = df$Species[1]
  
  points = df %>% dplyr::select(longitude,latitude)
  coordinates(points) = ~longitude+latitude
  crs(points) = "+proj=longlat +datum=WGS84"
  
  # project data to UTM
  points_laea = spTransform(points, proj_laea)
  
  # keep only points in the mask
  in.out = which(!is.na(extract(r_mask, points_laea)))
  points_laea <- points_laea[in.out,]
  
  # resample to the same number of points to control sampling effort
  points_laea_resampled = points_laea[sample(1:nrow(points_laea@coords),size=n_resample,replace=TRUE),]
  
  # resample to one point per grid cell
  points_laea_gridsampled = SpatialPoints(gridSample(points_laea_resampled, r_mask,n=1))
  crs(points_laea_gridsampled) = crs(points_laea_resampled)
  
  # count # of original and retained points
  n_raw = nrow(points@coords)
  n_resampled = nrow(points_laea_resampled@coords)
  n_gridsampled = nrow(points_laea_gridsampled@coords)
  
  # make random points
  points_random = SpatialPoints(randomPoints(mask=r_mask,
                                             n=n_gridsampled,
                                             p=points_laea_gridsampled,
                                             excludep=TRUE))
  crs(points_random) = crs(points_laea)
  
  # convert to pseudoabsence/presence data
  points_absence = SpatialPointsDataFrame(points_random,data.frame(PA=rep(0,n_gridsampled)))
  points_presence = SpatialPointsDataFrame(points_laea_gridsampled,data.frame(PA=rep(1,n_gridsampled)))
  points_final = rbind(points_absence, points_presence)
  
  # convert to SDM data format
  data_sdm = sdmData(formula=PA~., train=points_final, predictors=r_predictors)
  
  # return data
  return(list(data_sdm=data_sdm,
              points_final=points_final,
              n_raw=n_raw, 
              n_resampled=n_resampled,
              n_gridsampled=n_gridsampled,
              species_name=species_name))
}

occ_all = lapply(species_list, read.csv)

data_occurrences_25km = vector(mode="list",length=length(occ_all))
data_occurrences_50km = vector(mode="list",length=length(occ_all))
for (i in 1:length(occ_all))
{
  try(data_occurrences_25km[[i]] <- prepare_data(df=occ_all[[i]], 
                                            r_mask=r_na_25km, 
                                            r_predictors=r_climate_25km, 
                                            string_suffix="25km", 
                                            n_resample=500))
  
  try(data_occurrences_50km[[i]] <- prepare_data(df=occ_all[[i]], 
                                                 r_mask=r_na_50km, 
                                                 r_predictors=r_climate_50km, 
                                                 string_suffix="50km", 
                                                 n_resample=500))
  print(i)
}



data_gridsampled_25km = rbindlist(lapply(data_occurrences_25km, function(x) {
  if (!is.null(x))
  {
    pf = x$points_final
    w = subset(pf, PA==1)
    result = data.frame(w, species_name=x$species_name)
    return(result)
  }
  }))

data_gridsampled_50km = rbindlist(lapply(data_occurrences_50km, function(x) {
  if (!is.null(x))
  {
    pf = x$points_final
    w = subset(pf, PA==1)
    result = data.frame(w, species_name=x$species_name)
    return(result)
  }
}))

data_raw = rbindlist(occ_all,use.names=TRUE)
coordinates(data_raw) = ~longitude+latitude
crs(data_raw) = "+proj=longlat +datum=WGS84"
data_raw = spTransform(data_raw, crs(s_sonorandesert))
data_raw = as.data.frame(data_raw)






g_gridsample_25km_na = ggplot(data_gridsampled_25km,aes(x=x,y=y)) + 
  geom_bin2d(binwidth=c(25*1000,25*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_equal()  +
  ggtitle('Grid sampled, 25 km')

g_gridsample_25km_sd = ggplot(data_gridsampled_25km,aes(x=x,y=y)) + 
  geom_bin2d(binwidth=c(25*1000,25*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_fixed(ratio=1,xlim=c(extent(s_sonorandesert)@xmin,extent(s_sonorandesert)@xmax),ylim=c(extent(s_sonorandesert)@ymin,extent(s_sonorandesert)@ymax)) +
  ggtitle('Grid sampled, 25 km') +
  geom_point(data=df_s_sonorandesert,aes(x=long,y=lat),color='red',fill=NA,size=0.1)

g_gridsample_50km_na = ggplot(data_gridsampled_50km,aes(x=x,y=y)) + 
  geom_bin2d(binwidth=c(50*1000,50*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_equal() +
  ggtitle('Grid sampled, 50 km')

g_gridsample_50km_sd = ggplot(data_gridsampled_50km,aes(x=x,y=y)) + 
  geom_bin2d(binwidth=c(50*1000,50*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_fixed(ratio=1,xlim=c(extent(s_sonorandesert)@xmin,extent(s_sonorandesert)@xmax),ylim=c(extent(s_sonorandesert)@ymin,extent(s_sonorandesert)@ymax)) +
  ggtitle('Grid sampled, 50 km') +
  geom_point(data=df_s_sonorandesert,aes(x=long,y=lat),color='red',fill=NA,size=0.1)

g_raw_50km_na = ggplot(data_raw,aes(x=longitude,y=latitude)) + 
  geom_bin2d(binwidth=c(50*1000,50*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_equal() +
  ggtitle('Raw, 50 km')

g_raw_50km_sd = ggplot(data_raw,aes(x=longitude,y=latitude)) + 
  geom_bin2d(binwidth=c(50*1000,50*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_fixed(ratio=1,xlim=c(extent(s_sonorandesert)@xmin,extent(s_sonorandesert)@xmax),ylim=c(extent(s_sonorandesert)@ymin,extent(s_sonorandesert)@ymax)) +
  ggtitle('Raw, 50 km') +
  geom_point(data=df_s_sonorandesert,aes(x=long,y=lat),color='red',fill=NA,size=0.1)


g_raw_25km_na = ggplot(data_raw,aes(x=longitude,y=latitude)) + 
  geom_bin2d(binwidth=c(25*1000,25*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_equal() +
  ggtitle('Raw, 25 km')

g_raw_25km_sd = ggplot(data_raw,aes(x=longitude,y=latitude)) + 
  geom_bin2d(binwidth=c(25*1000,25*1000)) +
  scale_fill_continuous(type='viridis') +
  theme_void() +
  xlab("") + ylab("") +
  coord_fixed(ratio=1,xlim=c(extent(s_sonorandesert)@xmin,extent(s_sonorandesert)@xmax),ylim=c(extent(s_sonorandesert)@ymin,extent(s_sonorandesert)@ymax)) +
  ggtitle('Raw, 25 km') +
  geom_point(data=df_s_sonorandesert,aes(x=long,y=lat),color='red',fill=NA,size=0.1)

g_map = ggarrange(g_raw_25km_na, g_gridsample_25km_na, g_raw_50km_na, g_gridsample_50km_na, 
          g_raw_25km_sd, g_gridsample_25km_sd, g_raw_50km_sd, g_gridsample_50km_sd,
          nrow=2,ncol=4, common.legend = TRUE, legend='bottom',labels='AUTO'
          )

ggsave(g_map, width=12,height=6,file='g_map.png')
